# frozen_string_literal: true

Tagset.class_eval do # rubocop:disable Metrics/BlockLength
  field :operation_title, type: String
  field :type, type: String, default: 'directory'
  field :file_ext, type: String
  field :doc_file_size, type: Integer
  field :last_current_user_name, type: String
  field :current_state, type: String
  field :start_date, type: Date
  field :end_date, type: Date
  field :content_completeness, type: Integer, default: 100
  field :doc_ids, type: Array, default: []
  field :parent_tagset_ids, type: Array, default: []
  field :informed_names, type: Array, default: []
  field :informed_picture_urls, type: Array, default: []

  belongs_to :operation, index: true, optional: true, class_name: 'SpiritHub::Operation'

  before_create :spirit_hub_before_create
  before_save :spirit_hub_before_save
  after_save :update_children_parent_ids
  after_destroy :update_completeness

  def self.create_tagset_doc_from_file(files, user)
    doc_ids = []

    Array(files).each do |file|
      doc = Doc.create!(responsible: user.id, title: file.original_filename)
      doc.blob = file.read
      doc.update(type: 'file')
      doc_ids << doc.id
    end

    Tagset.create!(doc_ids:, responsible: user.id, type: 'document', title: Array(files).first.original_filename)
  end

  def doc?
    type == 'document'
  end

  def directory?
    type == 'directory'
  end

  def completed?
    return content_completeness == 100 if directory?

    true # if not a directory, it doesn't actually matter
  end

  def docs
    Doc.where(:id.in => doc_ids)
  end

  def doc
    Doc.find_by(id: doc_ids.first)
  end

  def children_tagsets
    Tagset.where(:id.in => Array(tagset_ids))
  end

  def parents
    Tagset.where(:id.in => Array(parent_tagset_ids))
  end

  def parent
    Tagset.find_by(id: parent_tagset_ids.to_a.first)
  end

  def completeness
    SpiritHub::Completeness.where(tagset_id: id)
  end

  def uncompleted_completeness
    SpiritHub::Completeness.where(tagset_id: id, completed: false)
  end

  def responsible_user
    User.find_by(id: responsible)
  end

  def current_user
    User.find_by(id: current_user_id)
  end

  def multiple_docs?
    doc_ids.to_a.length > 1
  end

  def update_docs_fields
    self.file_ext = case doc_ids.to_a.length
                    when 0
                      nil
                    when 1
                      docs.first.file_ext
                    else
                      'multiple'
                    end

    self.doc_file_size = doc_ids.to_a.empty? ? nil : docs.sum(:file_size)
  end

  def update_operation_title
    self.operation_title = operation&.title
  end

  def share_to_users(to, cc, bcc, current_user)
    all_users = User.where(:id.in => to.to_a + cc.to_a + bcc.to_a)

    upload = Upload.create(doc_id: doc_ids.first, user_id: current_user.id)

    all_users.each do |user|
      Agent.share(user.email, upload)&.deliver
    end

    update(informed: (informed.to_a + all_users.pluck(:id)).uniq)
    logs.create(action: 'Document partagé', code: 'CSL', user_id: current_user.id, data: { to:, cc:, bcc:, upload_id: upload.id })
  end

  def add_tagset_doc_to_object(object_id)
    if (tagset = Tagset.find_by(id: object_id))
      add_tagset_doc_to_tagset(tagset)
    elsif (completeness = SpiritHub::Completeness.find_by(id: object_id))
      add_tagset_doc_to_completeness(completeness)
    end

    if (new_state = cycle&.states&.find_by(key: :share))
      self.state_id = new_state.id
    end

    save
  end

  def remove_doc_from_tagset_doc(doc, user)
    doc.current_user_id = user.id
    doc.destroy
    doc_ids.delete(doc.id)
    save
  end

  def add_doc_to_tagset_doc(files, user)
    new_doc_ids = []

    Array(files).each do |file|
      new_doc_ids << Doc.create_from_file(file, user).id
    end

    self.doc_ids = doc_ids.to_a + new_doc_ids
    save
  end

  def purge!
    SpiritHub::TagsetPurger.call(self)
  end

  private

  def add_tagset_doc_to_completeness(completeness)
    completeness.update(object_ids: completeness.object_ids.to_a + [id])
    self.title = completeness&.title
    add_tagset_doc_to_tagset(completeness.tagset)
  end

  def add_tagset_doc_to_tagset(tagset)
    tagset.update(tagset_ids: tagset.tagset_ids.to_a + [id])
    self.operation_id = tagset.operation_id
  end

  def spirit_hub_before_create
    add_default_cycle if cycle_id.blank?
  end

  def add_default_cycle
    return unless (cycle = Cycle.find_by(key: 'hubdoc'))

    self.cycle_id = cycle.id
  end

  def spirit_hub_before_save
    update_operation_title if will_save_change_to_attribute?(:operation_id)
    update_docs_fields if will_save_change_to_attribute?(:doc_ids)
    update_informed_fields if will_save_change_to_attribute?(:informed)
    add_hubdoc_label if labels.exclude?('hubdoc')
  end

  def update_completeness
    SpiritHub::Completeness.where(object_ids: id).each do |completeness|
      completeness.update(object_ids: completeness.object_ids - [id])
    end
  end

  def update_informed_fields
    informed_users = User.where(:id.in => informed).order_by(first_name: :asc).only(:first_name, :last_name, :picture_url).to_a
    self.informed_names = informed_users.map(&:full_name)
    self.informed_picture_urls = informed_users.map(&:picture_url)
  end

  def add_hubdoc_label
    self.labels = labels.to_a + ['hubdoc']
  end

  def update_children_parent_ids
    old_tagset_ids = previous_changes['tagset_ids'].to_a.first || []
    new_tagset_ids = previous_changes['tagset_ids'].to_a.last || []

    to_remove = old_tagset_ids - new_tagset_ids
    to_add = new_tagset_ids - old_tagset_ids

    to_add.each do |child_id|
      Tagset.where(id: child_id).push(parent_tagset_ids: id)
    end

    to_remove.each do |child_id|
      Tagset.where(id: child_id).pull(parent_tagset_ids: id)
    end
  end
end
