# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(Tagset, type: :model) do
  describe 'add_default_cycle' do
    let!(:default_cycle) { create(:cycle, key: 'hubdoc') }

    it 'adds default cycle' do
      tagset = create(:tagset)
      expect(tagset.cycle_id).to eq(default_cycle.id)
    end

    it 'no error when no default cycle' do
      default_cycle.destroy
      tagset = create(:tagset)
      expect(tagset.cycle_id).to be_nil
    end
  end

  describe 'update_operation_title' do
    let!(:operation) { create(:operation, title: 'Operation test') }

    it 'updates operation title' do
      tagset = create(:tagset, operation:)
      expect(tagset.operation_title).to eq('Operation test')

      operation.update(title: 'Operation test 2')
      tagset.reload

      expect(tagset.operation_title).to eq('Operation test 2')
    end
  end

  describe 'update_informed_fields' do
    let!(:user1) { create(:user, first_name: 'User', last_name: '1') }
    let!(:user2) { create(:user, first_name: 'User', last_name: '2') }

    let!(:tagset) { create(:tagset) }

    it 'default' do
      expect(tagset.informed_names).to eq([])
      expect(tagset.informed_picture_urls).to eq([])
    end

    it 'add user' do
      tagset.update(informed: [user1.id.to_s])
      expect(tagset.informed_names).to eq(['User 1'])
      expect(tagset.informed_picture_urls).to eq([user1.reload.picture_url])
    end

    it 'add 2 users and remove them' do
      tagset.update(informed: [user1.id.to_s, user2.id.to_s])
      expect(tagset.informed_names).to eq(['User 1', 'User 2'])
      expect(tagset.informed_picture_urls).to eq([user1.reload.picture_url, user2.reload.picture_url])

      tagset.update(informed: [])
      expect(tagset.informed_names).to eq([])
      expect(tagset.informed_picture_urls).to eq([])
    end
  end

  describe 'add_hubdoc_label' do
    let!(:tagset) { create(:tagset) }

    it 'default' do
      expect(tagset.reload.labels).to include('hubdoc')
    end

    it 'dont add label if already present' do
      tagset.update(labels: %w[hubdoc other])
      expect(tagset.reload.labels).to eq(%w[hubdoc other])
    end
  end

  describe 'update_children_parent_ids' do
    let!(:tagset) { create(:tagset) }
    let!(:child1) { create(:tagset) }
    let!(:child2) { create(:tagset) }

    it 'default' do
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([])
      expect(child2.reload.parent_tagset_ids).to eq([])
    end

    it 'add child' do
      tagset.update(tagset_ids: [child1.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
    end

    it 'add 2 children and remove one' do
      tagset.update(tagset_ids: [child1.id.to_s, child2.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
      expect(child2.reload.parent_tagset_ids).to eq([tagset.id])

      tagset.update(tagset_ids: [child1.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
      expect(child2.reload.parent_tagset_ids).to eq([])
    end
  end

  describe 'update_completeness' do
    let!(:user) { create(:user) }
    let!(:tagset1) { create(:tagset) }
    let!(:tagset2) { create(:tagset) }
    let!(:completeness) { create(:completeness, tagset_id: tagset2.id, object_ids: [tagset1.id]) }

    it 'default' do
      expect(completeness.reload.object_ids).to eq([tagset1.id])
    end

    it 'when destroy tagset' do
      tagset1.current_user_id = user.id
      tagset1.destroy
      expect(completeness.reload.object_ids).to eq([])
    end
  end

  describe 'share_to_users' do
    let!(:tagset) { create(:tagset_doc, :with_file) }
    let!(:user1) { create(:user) }
    let!(:user2) { create(:user) }

    it 'shares to users' do
      expect { tagset.share_to_users([user1.id.to_s], [user2.id.to_s], [], user1) }.to change(Upload, :count).by(1)
      expect(tagset.reload.informed).to eq([user1.id, user2.id])
      expect(Log.count).to eq(1)
    end
  end

  describe 'update_docs_fields' do
    let!(:tagset) { create(:tagset) }

    it 'without docs' do
      expect(tagset.reload.file_ext).to be_nil
      expect(tagset.reload.doc_file_size).to eq(nil)
    end

    it 'with one doc' do
      doc = create(:doc, file_ext: 'jpeg', file_size: 1000)
      tagset.update(doc_ids: [doc.id])
      expect(tagset.reload.file_ext).to eq('jpeg')
      expect(tagset.reload.doc_file_size).to eq(1000)
    end

    it 'with multiple docs' do
      doc1 = create(:doc, file_ext: 'jpeg', file_size: 1000)
      doc2 = create(:doc, file_ext: 'pdf', file_size: 2000)
      tagset.update(doc_ids: [doc1.id, doc2.id])
      expect(tagset.reload.file_ext).to eq('multiple')
      expect(tagset.reload.doc_file_size).to eq(3000)
    end
  end

  describe 'purge!' do
    subject { tagset.purge! }

    let(:tagset) { create(:tagset_doc, current_user_id: user.id) }
    let(:user) { create(:user) }

    it 'destroys the record' do
      expect(tagset).to be_persisted
      subject
      expect(tagset).not_to be_persisted
      expect(Tagset.find(tagset.id)).to be_nil
    end

    context 'when the tagset is a child of another tagset' do
      let!(:tagset_parent1) { create(:tagset_directory, tagset_ids: ['other_tagset_id', tagset.id.to_s]) }
      let!(:tagset_parent2) { create(:tagset_directory, tagset_ids: [tagset.id.to_s]) }

      it 'clears the references from the parents' do
        expect(tagset_parent1.tagset_ids).to include(tagset.id.to_s)
        expect(tagset_parent2.tagset_ids).to include(tagset.id.to_s)

        subject

        expect(tagset_parent1.reload.tagset_ids).to eq ['other_tagset_id']
        expect(tagset_parent2.reload.tagset_ids).to eq []
      end
    end

    context 'when the tagset is a associated to a completeness rule' do
      let!(:completeness1) { create(:completeness, tagset: tagset_owner, object_ids: ['other_tagset_id', tagset.id]) }
      let!(:completeness2) { create(:completeness, tagset: tagset_owner, object_ids: [tagset.id]) }
      let(:tagset_owner) { create(:tagset_directory) }

      it 'clears the references from the parents' do
        expect(completeness1.object_ids).to include(tagset.id)
        expect(completeness2.object_ids).to include(tagset.id)

        subject

        expect(completeness1.reload.object_ids).to eq ['other_tagset_id']
        expect(completeness2.reload.object_ids).to eq []
      end
    end
  end

  describe 'create_tagset_doc_from_file' do
    let(:user) { create(:user) }
    let(:file1) { double('file1', original_filename: 'document1.pdf', read: 'File content 1') }
    let(:file2) { double('file2', original_filename: 'document2.pdf', read: 'File content 2') }

    context 'with multiple files' do
      it 'creates docs and a tagset with correct attributes' do
        files = [file1, file2]
        tagset = Tagset.create_tagset_doc_from_file(files, user)

        expect(tagset).to be_persisted
        expect(tagset.responsible).to eq(user.id)
        expect(tagset.type).to eq('document')
        expect(tagset.title).to eq(file1.original_filename)
        expect(tagset.doc_ids.size).to eq(files.size)

        tagset.doc_ids.each_with_index do |doc_id, index|
          doc = Doc.find(doc_id)
          file = files[index]

          expect(doc.responsible).to eq(user.id)
          expect(doc.title).to eq(file.original_filename)
          expect(doc.type).to eq('file')
        end
      end
    end

    context 'with a single file' do
      it 'creates a doc and a tagset with correct attributes' do
        files = file1

        tagset = Tagset.create_tagset_doc_from_file(files, user)

        expect(tagset.responsible).to eq(user.id)
        expect(tagset.type).to eq('document')
        expect(tagset.title).to eq(file1.original_filename)
        expect(tagset.doc_ids.size).to eq(1)

        doc = Doc.find(tagset.doc_ids.first)

        expect(doc.responsible).to eq(user.id)
        expect(doc.title).to eq(file1.original_filename)
        expect(doc.type).to eq('file')
      end
    end
  end

  describe 'add_tagset_doc_to_object' do
    let!(:cycle) { create(:cycle) }
    let!(:state) { create(:state, key: :share, cycle_id: cycle.id) }
    let!(:model) { create(:tagset, cycle_id: cycle.id) }
    let!(:operation) { create(:operation, title: 'Operation 1') }
    let(:tagset) { create(:tagset, operation_id: operation.id) }
    let(:completeness) { create(:completeness, title: 'Completness', tagset_id: tagset.id) }

    context 'when a Tagset is found by object_id' do
      it 'add the model to the tagset' do
        model.add_tagset_doc_to_object(tagset.id)

        expect(model.state_id).to eq(state.id)
        expect(model.operation_id).to eq(tagset.operation_id)
        expect(model.operation_title).to eq(tagset.operation_title)
        expect(tagset.reload.tagset_ids).to include(model.id)
      end
    end

    context 'when a Completeness is found by object_id' do
      it 'add the model to the completeness' do
        model.add_tagset_doc_to_object(completeness.id)

        expect(model.state_id).to eq(state.id)
        expect(model.title).to eq(completeness.title)
        expect(model.operation_id).to eq(tagset.operation_id)
        expect(model.operation_title).to eq(tagset.operation_title)
        expect(completeness.reload.object_ids).to include(model.id)
        expect(tagset.reload.tagset_ids).to include(model.id.to_s)
      end
    end
  end

  describe 'add_doc_to_tagset_doc' do
    let!(:user) { create(:user) }
    let!(:tagset_doc) { create(:tagset_doc) }
    let(:file1) { double('file1', original_filename: 'document1.pdf', read: 'File content 1') }
    let(:file2) { double('file2', original_filename: 'document2.pdf', read: 'File content 2') }

    it 'with multiple files' do
      files = [file1, file2]
      tagset_doc.add_doc_to_tagset_doc(files, user)

      expect(tagset_doc.doc_ids.size).to eq(files.size)

      tagset_doc.doc_ids.each_with_index do |doc_id, index|
        doc = Doc.find(doc_id)
        file = files[index]

        expect(doc.responsible).to eq(user.id)
        expect(doc.title).to eq(file.original_filename)
        expect(doc.type).to eq('file')
      end
    end

    it 'with a single file' do
      tagset_doc.add_doc_to_tagset_doc(file1, user)
      expect(tagset_doc.doc_ids.size).to eq(1)

      doc = Doc.find(tagset_doc.doc_ids.first)

      expect(doc.responsible).to eq(user.id)
      expect(doc.title).to eq(file1.original_filename)
      expect(doc.type).to eq('file')
    end
  end

  describe 'remove_doc_from_tagset_doc' do
    let!(:user) { create(:user) }
    let!(:tagset_doc) { create(:tagset_doc, :with_file) }

    it 'works' do
      doc = Doc.find(tagset_doc.doc_ids.first)
      tagset_doc.remove_doc_from_tagset_doc(doc, user)

      expect(tagset_doc.doc_ids).to eq([])
      expect(Doc.find_by(id: doc.id)).to be_nil
    end
  end
end
